%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Code for the Firm Entry/Exit Problem presented in the paper
% "Identification of Counterfactuals in Dynamic Discrete Choice Models,"
% by Myrto Kalouptsidi, Paul Scott, and Eduardo Souza-Rodrigues

% This Version: February 2020

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% (A) Parameters

% State Variables: (k,w)
% X=6 possible states:{(k=0,w=20) (k=0,w=17) (k=0,w=12) (k=1,w=20) (k=1,w=17) (k=1,w=12)}

% Static Parameters
c   = 11;            % Marginal cost
eta = 1.5;           % Demand slope
w   = [20 17 12];    % Demand shocks: high, medium, and low 

% Dynamic Parameters
beta = 0.95;            % Discount factor
fc   = [5.5 5.5 5.5];   % Fixed costs
sv   = [10 10 10];      % Scrap values
ec   = [9 9 9];         % Entry costs

% Flow Payoffs: profit(a,k,w)
profit = zeros(2,6);

% Payoffs for a = 0, i.e. being inactive
profit(1,1:3) = 0;          % Stay Out
profit(1,4:6) = sv(1,:);    % Exit

% Payoff for a = 1, i.e. being active
profit(2,1:3) = -ec(1,:);                   % Enter
profit(2,4:6) = ((w-c).^2)./(4*eta) - fc;   % Stay In

% Transition Matrix for Demand Shocks, Fw 
f = [0.4 0.3 0.2; 0.35 0.4 0.2; 0.25 0.3 0.6];

% Full Transition Matrix F
F(:,:,1) = [f f; zeros(3,6)];     % transition matrix for a=0, i.e. inactive
F(:,:,2) = [zeros(3,6); f f];     % transition matrix for a=1, i.e. active

% Euler Constant
eulergamma = 0.5772156649;


%% (B) Solve for the *TRUE BASELINE* Value Function and CCP

% Initial guess for the ex ante Value Function
V0 = zeros(1,6);

% True Value Function
V_true = solvevalue(profit,beta,F,V0);

% Conditional Choice Probabilities (CCP) and Conditional Value (v)
[p_true, cv_true] = cprobs(profit,beta,F,V_true);


%% (C) Estimated Payoffs under Alternative Identifying Restrictions

% For all cases, profit of staying out is known, profit(1,1:3) = 0.

% Relevant matrices and vectors used in the calculations 
% (from equations (3)--(5) in main text)
A2    = (eye(6)-beta*F(:,:,2)') * inv(eye(6)-beta*F(:,:,1)');   % equation (4)
psi_1 = log(1 + p_true(2,:) ./ p_true(1,:)) + eulergamma;
psi_2 = log(1 + p_true(1,:) ./ p_true(2,:)) + eulergamma;

%%% (C1) Restricted Model #1: Set sv = 0 in all states
profit_n1(1,:) = zeros(1,6);
profit_n1(2,:) = profit_n1(1,:)*A2' + psi_1*A2' - psi_2; % equation (3)

% Check: Solve for the baseline case using estimated payoffs
V_n1         = solvevalue(profit_n1,beta,F,V0);
[p_n1 cv_n1] = cprobs(profit_n1,beta,F,V_n1);

%%% (C2) Restricted Model #2: Set fc = 0 in all states

%%% OBS 1: Assume profit(2,4:6) = ((w-c).^2)./(4*eta) is known.  
%%% OBS 2: Need to solve an augmented system of linear equations: add
%%% restrictions to the system of equations (3), so that [profit(2),profit(1)] * Y = y

% Prepare Vectors
b2 = (psi_1*A2' - psi_2)';                      % b vector from equation (5)
r  = [profit(1,1:3)'; (((w-c).^2)./(4*eta))'];  % used for the additional restrictions
y  = [b2; r];

% Prepare Matrices
A = [-A2,eye(6)];
R = zeros(6,12);
R(1:3,1:3)   = eye(3);      % known outside option: profit(1,1:3) 
R(4:6,10:12) = eye(3);      % known variable profit: profit(2,4:6)
Y = [A;R];                  % matrix of the augmented system

% Solve the Augmented System of Equations
x = Y\y;

% Estimated Payoffs
profit_n2(1,:) = x(1:6)';
profit_n2(2,:) = x(7:12)';

% Check: Solve for the baseline case using estimated payoffs
V_n2         = solvevalue(profit_n2,beta,F,V0);
[p_n2 cv_n2] = cprobs(profit_n2,beta,F,V_n2);


%% (D) Counterfactual #1: Additive Entry Subsidy 

%%% (D1) Set Counterfactual Subsidies
s1 = 0.9;

%%% (D2) True Model
% Counterfactual Payoffs
profit_cf1        = profit;                       
profit_cf1(2,1:3) = profit_cf1(2,1:3) + s1; 

% Counterfactual CCPs and Values
V_cf1       = solvevalue(profit_cf1,beta,F,V0);
[p_cf1 cv_cf1] = cprobs(profit_cf1,beta,F,V_cf1);

%%% (D3) Restricted Model #1: Set sv = 0 in all states
% Counterfactual Payoffs
profit_n1_cf1        = profit_n1;                      
profit_n1_cf1(2,1:3) = profit_n1(2,1:3) + s1;

% Counterfactual CCPs and Values
V_n1_cf1             = solvevalue(profit_n1_cf1,beta,F,V0);
[p_n1_cf1 cv_n1_cf1] = cprobs(profit_n1_cf1,beta,F,V_n1_cf1);

%%% (D4) Restricted Model #2: Set fc = 0 in all states
% Counterfactual Payoffs
profit_n2_cf1        = profit_n2;
profit_n2_cf1(2,1:3) = profit_n2(2,1:3) + s1;

% Counterfactual CCPs and Values
V_n2_cf1             = solvevalue(profit_n2_cf1,beta,F,V0);
[p_n2_cf1 cv_n1_cf1] = cprobs(profit_n2_cf1,beta,F,V_n2_cf1);


%% (E) Counterfactual #2: Proportional Entry Subsidy

%%% (E1) Set Counterfactual Subsidies
s2 = 0.9;

%%% (E2) True Model
% Counterfactual Payoffs
profit_cf2        = profit;
profit_cf2(2,1:3) = s2 * profit_cf2(2,1:3);

% Counterfactual CCPs and Values
V_cf2       = solvevalue(profit_cf2,beta,F,V0);
[p_cf2 cv_cf2] = cprobs(profit_cf2,beta,F,V_cf2);

%%% (E3) Restricted Model #1: Set sv = 0 in all states
% Counterfactual Payoffs
profit_n1_cf2        = profit_n1;                      
profit_n1_cf2(2,1:3) = s2 * profit_n1(2,1:3);

% Counterfactual CCPs and Values
V_n1_cf2             = solvevalue(profit_n1_cf2,beta,F,V0);
[p_n1_cf2 cv_n1_cf2] = cprobs(profit_n1_cf2,beta,F,V_n1_cf2);

%%% (E4) Restricted Model #2: Set fc = 0 in all states
% Counterfactual Payoffs
profit_n2_cf2        = profit_n2;
profit_n2_cf2(2,1:3) = s2 *profit_n2(2,1:3);

% Counterfactual CCPs and Values
V_n2_cf2             = solvevalue(profit_n2_cf2,beta,F,V0);
[p_n2_cf2 cv_n1_cf2] = cprobs(profit_n2_cf2,beta,F,V_n2_cf2);


%% (F) Counterfactual #3: Change in Transitions

%%% (F1) Set Counterfactual Transition Matrix
f_cf = [1/3 1/3 1/3; 1/3 1/3 1/3; 1/3 1/3 1/3];    % counterfactual matrix Fw
F_cf(:,:,1) = [f_cf f_cf; zeros(3,6)];           % transition matrix for a=0 i.e. inactive
F_cf(:,:,2) = [zeros(3,6); f_cf f_cf];           % transition matrix for a=1 i.e. active

%%% (F2) True Model
% Counterfactual CCPs and Values
V_cf3       = solvevalue(profit,beta,F_cf,V0);
[p_cf3 cv_cf3] = cprobs(profit,beta,F_cf,V_cf3);

%%% (F3) Restricted Model #1: Set sv = 0 in all states
% Counterfactual CCPs and Values
V_n1_cf3             = solvevalue(profit_n1,beta,F_cf,V0);
[p_n1_cf3 cv_n1_cf3] = cprobs(profit_n1,beta,F_cf,V_n1_cf3);

%%% (F4) Restricted Model #2: Set fc = 0 in all states
% Counterfactual CCPs and Values
V_n2_cf3             = solvevalue(profit_n2,beta,F_cf,V0);
[p_n2_cf3 cv_n2_cf3] = cprobs(profit_n2,beta,F_cf,V_n2_cf3);


%% (G) Counterfactual #4: Change in Types (Markets Entry Costs) 

% Replace Entry Cost of Market Type 1 by Market Type 2

%%% (G1) Set Counterfactual Change in Entry Costs

% Parameters for Market Type 2
% Static Parameters
c2   = 9;             % Marginal cost
eta2 = 1.7;           % Demand slope
w2   = [18 15 11];    % Demand shocks: high, medium, and low

% Dynamic Parameters
fc2  = [3 3 3];       % Fixed costs
sv2  = [8 8 8];       % Scrap values
ec2  = [6 6 6];       % Entry costs

% True Payoff
profit2        = zeros(2,6);
profit2(1,4:6) = sv2(1,:);                      % Exit
profit2(2,1:3) = -ec2(1,:);                     % Enter
profit2(2,4:6) = ((w2-c2).^2)./(4*eta2) - fc2;  % Stay In

% Solve True Value Functions and CCPs for Market Type 2
V2_true             = solvevalue(profit2,beta,F,V0);
[p_true2, cv_true2] = cprobs(profit2,beta,F,V2_true);

% Find Payoffs under Alternative Identifying Restrictions for Market Type 2
% Set Relevant vectors
psi2_1 = log(1 + p_true2(2,:) ./ p_true2(1,:)) + eulergamma;
psi2_2 = log(1 + p_true2(1,:) ./ p_true2(2,:)) + eulergamma;

% Restricted Model #1: Set sv = 0 in all states
% Estimated Payoffs in Market Type 2
profit2_n1(1,:) = zeros(1,6);
profit2_n1(2,:) = profit2_n1(1,:)*A2' + psi2_1*A2' - psi2_2;    % equation (3)

% Restricted Model #2: Set fc = 0 in all states
% As before, need to solve an augmented system of equations

% Prepare Vectors and Matrices
b22 = (psi2_1*A2' - psi2_2)';
r2  = [profit2(1,1:3)'; (((w2-c2).^2)./(4*eta2))'];
y2  = [b22; r2];
Y2  = [A;R];

% Solve the System of Equations
x2 = Y2\y2;

% Estimated Payoffs in Market Type 2
profit2_n2(1,:) = x2(1:6)';
profit2_n2(2,:) = x2(7:12)';


%%% (G2) True Model
% Counterfactual Payoffs
profit_cf4        = profit;
profit_cf4(2,1:3) = profit2(2,1:3); 

% Counterfactual CCPs and Values
V_cf4          = solvevalue(profit_cf4,beta,F,V0);
[p_cf4 cv_cf4] = cprobs(profit_cf4,beta,F,V_cf4);

%%% (G3) Restricted Model #1: Set sv = 0 in all states
% Counterfactual Payoffs
profit_n1_cf4        = profit_n1;                      
profit_n1_cf4(2,1:3) = profit2_n1(2,1:3);

% Counterfactual CCPs and Values
V_n1_cf4 = solvevalue(profit_n1_cf4,beta,F,V0);
[p_n1_cf4 cv_n1_cf4] = cprobs(profit_n1_cf4,beta,F,V_n1_cf4);



%%% (G4) Restricted Model #2: Set fc = 0 in all states
% Counterfactual Payoffs
profit_n2_cf4        = profit_n2;                      
profit_n2_cf4(2,1:3) = profit2_n2(2,1:3);

% Counterfactual CCPs and Values
V_n2_cf4 = solvevalue(profit_n2_cf4,beta,F,V0);
[p_n2_cf4 cv_n2_cf4] = cprobs(profit_n2_cf4,beta,F,V_n2_cf4);


%% (H) SHOW RESULTS

%%% (H1) Table 1
disp('***************************************************************************');
disp('TABLE 1: PAYOFF FUNCTIONS');
disp('***************************************************************************');
%
disp('1st line: a = 0 (inactive)| 2nd line: a = 1 (active)');
disp('States:{(k=0,w=w_h) (k=0,w=w_m) (k=0,w=w_l) (k=1,w=w_h) (k=1,w=w_m) (k=1,w=w_l)}');
disp('***************************************************************************');
%
disp('True Payoff Function');
disp('***************************************************************************');
disp(profit);
%
disp('***************************************************************************');
disp('Payoff Model 1: Scrap Value = 0');
disp('***************************************************************************');
disp(profit_n1);
%
disp('***************************************************************************');
disp('Payoff Model 2: Fixed Cost = 0');
disp('***************************************************************************');
disp(profit_n2);
pause

%%% (H2) Table 2
% COUNTERFACTUALS
disp('')
disp('')
%
disp('***************************************************************************');
disp('TABLE 2: COUNTERFACTUAL 1, Additive Entry Subsidy');
disp('***************************************************************************');
%
disp('Choice Probabilities - Pr(active|x)');
disp('Baseline vs True CF vs Model 1 (sv=0) vs Model 2 (fc=0)');
disp('***************************************************************************');
disp([reshape(100*p_true(2,:),6,1) reshape(100*p_cf1(2,:),6,1) reshape(100*p_n1_cf1(2,:),6,1) reshape(100*p_n2_cf1(2,:),6,1)]);
%
disp('***************************************************************************');
disp('Change in ex-ante Value function: V_cf - V');
disp('True vs Model 1 (sv=0) vs Model 2 (fc=0)');
disp('***************************************************************************');
disp([reshape(V_cf1 - V_true,6,1) reshape(V_n1_cf1 - V_n1,6,1) reshape(V_n2_cf1 - V_n2,6,1)]);
pause
disp('***************************************************************************');
disp('TABLE 2: COUNTERFACTUAL 2, Proportional Entry Subsidy');
disp('***************************************************************************');
%
disp('Choice Probabilities - Pr(active|x)');
disp('Baseline vs True CF vs Model 1 (sv=0) vs Model 2 (fc=0)');
disp('***************************************************************************');
disp([reshape(100*p_true(2,:),6,1) reshape(100*p_cf2(2,:),6,1) reshape(100*p_n1_cf2(2,:),6,1) reshape(100*p_n2_cf2(2,:),6,1)]);
%
disp('***************************************************************************');
disp('Change in ex-ante Value function: V_cf - V');
disp('True vs Model 1 (sv=0) vs Model 2 (fc=0)');
disp('***************************************************************************');
disp([reshape(V_cf2 - V_true,6,1) reshape(V_n1_cf2 - V_n1,6,1) reshape(V_n2_cf2 - V_n2,6,1)]);
pause

%%% (H3) Table 3
%
disp('***************************************************************************');
disp('TABLE 3: COUNTERFACTUAL 3, Change in Transitions');
disp('***************************************************************************');
%
disp('Choice Probabilities - Pr(active|x)');
disp('Baseline vs True CF vs Model 1 (sv=0) vs Model 2 (fc=0)');
disp('***************************************************************************');
disp([reshape(100*p_true(2,:),6,1) reshape(100*p_cf3(2,:),6,1) reshape(100*p_n1_cf3(2,:),6,1) reshape(100*p_n2_cf3(2,:),6,1)]);
%
disp('***************************************************************************');
disp('Change in ex-ante Value function: V_cf - V');
disp('True vs Model 1 (sv=0) vs Model 2 (fc=0)');
disp('***************************************************************************');
disp([reshape(V_cf3 - V_true,6,1) reshape(V_n1_cf3 - V_n1,6,1) reshape(V_n2_cf3 - V_n2,6,1)]);
pause
disp('***************************************************************************');
disp('TABLE 3: COUNTERFACTUAL 4, Change Market Entry Costs');
disp('***************************************************************************');
%
disp('Choice Probabilities - Pr(active|x)');
disp('Baseline vs True CF vs Model 1 (sv=0) vs Model 2 (fc=0)');
disp('***************************************************************************');
disp([reshape(100*p_true(2,:),6,1) reshape(100*p_cf4(2,:),6,1) reshape(100*p_n1_cf4(2,:),6,1) reshape(100*p_n2_cf4(2,:),6,1)]);
%
disp('***************************************************************************');
disp('Change in ex-ante Value function: V_cf - V');
disp('True vs Model 1 (sv=0) vs Model 2 (fc=0)');
disp('***************************************************************************');
disp([reshape(V_cf4 - V_true,6,1) reshape(V_n1_cf4 - V_n1,6,1) reshape(V_n2_cf4 - V_n2,6,1)]);

