
set more off

local mainfolder = "/home/landvalues/"


* --------------------------------------------------------------------------------
* clean data

use "`mainfolder'merged_data.dta", clear

* generate j and k
destring cdl*, force replace
do "`mainfolder'classifyCropscape.do"

* generate k
forvalues yr=2008/2014 {
	local lyr = `yr' - 1
	local l2yr = `yr'-2
	gen k`yr'=0 if j`lyr'==1
	replace k`yr' = 1 if j`lyr'==0 & j`l2yr'==1
	replace k`yr' = 2 if j`lyr'==0 & j`l2yr'==0
}


* bring in expected returns data
drop fipsstate
drop fipscounty
gen fips = stfips*1000 + cofips
merge m:1 fips using "`mainfolder'r.dta"
drop if _merge==2
drop _merge

* find the k and j that corrseponds to the transaction year
gen currentk = .
local yr 2008 2009 2010 2011 2012 2013 2014
foreach y in `yr' {
	replace currentk =  k`y' if transfer_year == `y'
}

gen currentj = .
local yr 2006 2007 2008 2009 2010 2011 2012 2013 2014
foreach y in `yr' {
	replace currentj =  j`y' if transfer_year == `y'
}

gen currentR = .
local yr 2006 2007 2008 2009 2010 2011 2012 2013
foreach y in `yr' {
	replace currentR = r`y' if transfer_year == `y'
}

* flag properties if land use is ever in non-modeled category
gen useflag = 0
local yr 2006 2007 2008 2009 2010 2011 2012 2013 2014
foreach y in `yr' {
	replace useflag = 1 if j`y'==8
}

* we don't have k for properties which choose non-modeled land uses like developed land
* drop if missing(currentk)
drop if useflag
drop useflag

rename altitude altitude_crude
rename srtmaltitude alt

* we're only interested in transactions in years
* where we know k
drop if transfer_year < 2008

* drop missing/zero prices
drop if missing(resalepricefinal)

* drop if we don't know the year of the sale transaction
drop if missing(transfer_year)

* there are values in altitude that are "NA". Get rid of them
drop if alt == "NA"
destring(alt), replace
destring(slope), replace

* NOTE: some missing values of aspect remain. if we decide it is important, we might
* want to drop those observations

* we can't hope to do this estimation for counties where we don't have a measure of r
drop if missing(currentR)

* basic soil types
drop if gaezsoil_dominant=="NA"
destring gaezsoil_dominant, replace
rename gaezsoil_dominant soil

* we need k for VF estimation
drop if missing(currentk)

tab transfer_year currentk

* --------------------------------------------------------------------------------------
* some variables which are useful for hedonic regressions
* note: we should create nonlinear variables and interactions before collapsing multi-parcel transactions

gen year = floor(transfer_date/10000)
gen month = floor(transfer_date/100) - year*100

destring aspect, force replace
replace aspect = 0 if aspect==. & slope==0
gen north = abs(aspect-_pi)
gen adjslope = north*slope

rename resalepricefinal v

destring gaezpop_dens, replace
destring gaezworkability, force replace
destring gaezshare_cultivated, force replace

gen la = log(acres)
encode use_code, gen(dquse)

* higher-order variables
gen slope2 = slope^2
gen latslope = lat*adjslope

* fix longitude sign
replace lon = -lon

save "`mainfolder'clean_data_uncollapsed.dta", replace


* --------------------------------------------------------------------------------------
* ORGANIZE MULTI-PARCEL TRANSACTIONS
egen saleid = group(mailaddr transfer_val transfer_date)
drop if saleid == .

* within the transactions not listed as multi-parcel transactions, there are some transactions which look suspiciously linked,
* involving the same buyer, price, and transaction date. Some of these could reflect genuinely separate properties with identical
* terms, but many of them may have been recorded improperly. Drop them.
bysort saleid: gen nparcels = _N
drop if nparcels > 1 & multiple_parcels == ""

* for multi-parcel transactions, collapse by:
* -- taking the weighed mean of all continuous characteristics
* -- taking the value from the largest parcel for categorical variables

* variables we're taking the mean of for multi-parcel transactions
local meanvars = "aspect north slope adjslope alt slope2 latslope  "

* variables we're taking the sum of
local addvars = "acres assessedmarketvalue assessedmarketlandvalue"

destring gaezprotected trust disabledexemption seniorexemption veteranexemption widowexemption otherexemption, force replace

* variables we're taking the value from the largest parcel
local ivars = "fips lon lat zip currentj currentk currentR  gaezpop_dens dquse soil geoqlty no_money  ownertype trust transfer_val  gaezshare_cultivated gaezworkability transaction_quit_claim "
local ivars2 = "cdl2006 cdl2007 cdl2008 cdl2009 cdl2010 cdl2011 cdl2012 cdl2013 cdl2014 j2006 j2007 j2008 j2009 j2010 j2011 j2012 j2013 j2014 k2008 k2009 k2010 k2011 k2012 k2013 k2014 "
local ivars3 = "disabledexemption disabledexemption_amt seniorexemption seniorexemption_amt veteranexemption veteranexemption_amt widowexemption widowexemption_amt otherexemption otherexemption_amt "

* string variables
tostring dqid, replace
local svars = "dqid use_code buyer seller transaction_type transfer_filing township_as range_as section_as quarter_as tract_as unit_as distress gaezprotected  "

gen largest = 1 if nparcels ==1
sort saleid acres
replace largest = 1 if saleid[_n+1] != saleid
replace largest = 0 if largest==.

foreach x in `ivars' `ivars2' `ivars3' `svars' {
	gen `x'_largest = `x' if largest
	drop `x'
	rename `x'_largest `x'
}

gen weight = round(acres)
collapse (mean) v `meanvars' (firstnm) `ivars' `ivars2' `svars' nparcels (sum) `addvars' [fw = weight], by(saleid transfer_date year month)
destring dqid, replace

* --------------------------------------------------------------------------------------
* more useful variables
gen multi = 1 if nparcels>1
replace multi = 0 if nparcels==1

* year of planting season, assuming spring planting
gen pyear = year if month<=4
replace pyear = year+1 if month>4

* log land price
gen lv = log(v)

* free up some space and save
compress
save "`mainfolder'clean_data.dta", replace
