#! /usr/bin/Rscript

## Author: Adrian Torchiana (adrian.torchiana@gmail.com)
## RA work for Paul Scott (ptscott@gmail.com)

## example usage: nohup ./spatial_merge.R --dq.sample.first=1 --dq.sample.size=1000 --output.file=output_1k_with_aspect.csv > log_aspect2.txt &


library(maptools)
library(optparse)  # Command line options for Rscript
library(plyr)
library(raster)
## library(rgeos)
library(rgdal)  # On Ubuntu I had to first install libgdal-dev and libproj-dev
library(sp)
library(stringr)

source("fns_spatial_merge.R")

opt.list <- list(make_option("--data.dir", default="/home/landvalues/spatial_data",
                             help="Directory containing dq_points1.csv, weather_grid.csv, GAEZ, CDL, etc"),
                 make_option("--dq.file", default="lv_centroids.csv",
                             help="Name of file containing dq points"),
                 make_option("--dq.sample.first", default=1,
                             help="first row to start sampling"),
                 make_option("--dq.sample.size", default=1000,
                             help="number of consecutive rows to sample"),
                 make_option("--neighbor.distance", default=5,
                             help="Arc second distance for slope and aspect calculations"),
                 make_option("--output.file", default="example_output.csv", help="File in which to save output"))
opt <- parse_args(OptionParser(option_list=opt.list))
message("Command line options: ", paste(sprintf("%s=%s", names(opt), opt), collapse=", "))
stopifnot(opt$neighbor.distance >= 1)

filename <- file.path(opt$data.dir, opt$dq.file)
message("Loading ", filename)
dq <- read.csv(filename, stringsAsFactors=FALSE)
stopifnot(all(c("dqid", "lon", "lat") %in% names(dq)))
message("Loaded ", nrow(dq), " rows and ", ncol(dq), " columns")
message(length(unique(dq$dqid)), " unique dqid")

dq.sample.last = opt$dq.sample.first + opt$dq.sample.size - 1
 
if (!is.null(opt$dq.sample.size)) {
    message("sampling ", opt$dq.sample.size, " rows of dq")
##  dq <- dq[sample(1:nrow(dq), size=opt$dq.sample.size, replace=FALSE), ]
 	dq <- dq[opt$dq.sample.first:dq.sample.last, ]
    message("Now ", nrow(dq), " rows and ", ncol(dq), " columns")
    message(length(unique(dq$dqid)), " unique dqid")
}

filename <- file.path(opt$data.dir, "weather_grid.csv")
message("Loading ", filename)
wg <- read.csv(filename, stringsAsFactors=FALSE)
stopifnot(all(c("srid", "lon", "lat") %in% names(wg)))
message("Loaded ", nrow(wg), " rows and ", ncol(wg), " columns")
message(length(unique(wg$srid)), " unique srid")
message(length(unique(wg$lat)), " unique lat; ", length(unique(wg$lon)), " unique lon")

coords <- c("lon", "lat")  # Column names
dq.pts <- SpatialPointsDataFrame(coords=dq[, coords], data=dq, proj4string=CRS("+proj=longlat +datum=WGS84"))
wg.pts <- SpatialPointsDataFrame(coords=wg[, coords], data=wg, proj4string=CRS("+proj=longlat +datum=WGS84"))

message("Calling summary(dq.pts)")
summary(dq.pts)
## plot(dq.pts)
## plot(wg.pts)

filename <- file.path(opt$data.dir, "tl_2010_us_county10", "tl_2010_us_county10.shp")
message("Loading ", filename)
counties <- readShapeSpatial(filename, proj4string=CRS("+proj=longlat +datum=WGS84"))
counties$fips.state <- as.numeric(as.character(counties$STATEFP10))  # Careful converting factors
counties$fips.county <- as.numeric(as.character(counties$COUNTYFP10))
counties$name <- as.character(counties$NAMELSAD10)
counties.48 <- counties[counties$fips.state < 60 &  # Samoa, Guam, Puerto Rico
                        counties$fips.state != 2 &  # Alaksa
                        counties$fips.state != 15, ]  # Hawaii
## plot(counties.48)  # Sanity check
stopifnot(length(unique(counties.48$fips.state)) == 48 + 1)  # 48 states plus DC

message("Overlaying county data onto dq.pts")
dq.counties.48.overlay <- over(dq.pts, counties.48)
dq.pts <- spCbind(dq.pts, dq.counties.48.overlay[, c("fips.state", "fips.county", "name")])

message("Removing dq.pts outside of 48 + DC")
in.lower.48 <- !is.na(dq.pts$fips.state) & !is.na(dq.pts$fips.county)
dq.pts <- dq.pts[in.lower.48, ]
message("Now ", nrow(dq.pts), " rows and ", ncol(dq.pts), " columns")
message(length(unique(dq.pts$dqid)), " unique dqid")
stopifnot(nrow(dq.pts) > 0)
## plot(dq.pts)  # Sanity check

## Neighboring points used in slope and aspect calculations
degree.deltas <- c(opt$neighbor.distance, 0, -opt$neighbor.distance) / (60 * 60)
neighbor.deltas <- expand.grid(lon=degree.deltas, lat=degree.deltas)
neighbor.labels <- expand.grid(lon=c("E", "", "W"), lat=c("N", "", "S"))
neighbor.deltas$label <- sprintf("%s%s", neighbor.labels$lat, neighbor.labels$lon)
neighbor.deltas <- neighbor.deltas[neighbor.deltas$lat != 0 | neighbor.deltas$lon != 0, ]
stopifnot(nrow(neighbor.deltas) == 8)  # Neighbors to NW, N, NE, E, SE, S, SW, W
dq.neighbors <- data.frame(dqid=rep(dq.pts$dqid, rep(8, nrow(dq.pts))))
dq.neighbors$lon <- rep(dq.pts$lon, rep(8, nrow(dq.pts))) + rep(neighbor.deltas$lon, nrow(dq.pts))
dq.neighbors$lat <- rep(dq.pts$lat, rep(8, nrow(dq.pts))) + rep(neighbor.deltas$lat, nrow(dq.pts))
dq.neighbors$label <- rep(neighbor.deltas$label, nrow(dq.pts))
dq.neighbors <- SpatialPointsDataFrame(coords=dq.neighbors[, coords], data=dq.neighbors,
                                       proj4string=CRS("+proj=longlat +datum=WGS84"))
stopifnot(nrow(dq.neighbors) == 8 * nrow(dq.pts))

for (var in list.files(file.path(opt$data.dir, "GAEZ"))) {
    varname <- sprintf("gaez.%s", var)
    filename <- file.path(opt$data.dir, "GAEZ", var, "data.asc")
    if (file.exists(filename)) {
        dq.pts@data[, varname] <- get.raster.variable(filename)
    } else {
        message("Could not find ", filename, "; skipping")
    }
}

cdl.files <- list.files(file.path(opt$data.dir, "CDL"))
cdl.files <- cdl.files[str_detect(cdl.files, "^cdl_[0-9]{2}m_r_[a-z]{2}_[0-9]{4}_albers.tif$")]
message("Found ", length(cdl.files), " CDL files matching regex")
cdl.years <- sort(unique(str_extract(cdl.files, "[0-9]{4}")))
stopifnot(all(cdl.years >= 1950) && all(cdl.years <= 2050))  # Sanity check
message("CDL years : ", paste(cdl.years, collapse=", "))
for (year in cdl.years) {
    year.regex <- sprintf("_%s_albers.tif$", year)
    year.files <- cdl.files[str_detect(cdl.files, year.regex)]  # Should find several states for given year
    message("Found ", length(year.files), " CDL file(s) for ", year)
    varname <- sprintf("cdl.%s", year)
    raster.values <- rep(NA, nrow(dq.pts))  # Contains values for all states seen so far
    for (filename in year.files) {
        filename <- file.path(opt$data.dir, "CDL", filename)
        raster.values.state <- get.raster.variable(filename)  # Will be NA for points outside of state
        ## stopifnot(all(is.na(raster.values) | is.na(raster.values.state)))  # TODO Points near state border?
raster.values[!is.na(raster.values.state) & raster.values.state != 0] <- raster.values.state[!is.na(raster.values.state) & raster.values.state != 0]    }
    dq.pts@data[, varname] <- raster.values
}

message("Assigning altitudes using getData('alt', country='USA') from raster package")
tryCatch({
	altitudes <- getData("alt", country="USA")  # Returns a list of rasters
	stopifnot(is.list(altitudes) && length(altitudes) == 4)
	dq.pts$altitude <- extract(altitudes[[1]], dq.pts)  # First element of altitudes covers 48 + DC
##	spplot(dq.pts, "altitude")  # Sanity check
})

## success = 0
## counter = 0
## while (success==0) {
## 	result = tryCatch({
## 		message("Assigning altitudes using getData('alt', country='USA') from raster package")
## 		altitudes <- getData("alt", country="USA")  # Returns a list of rasters
## 		stopifnot(is.list(altitudes) && length(altitudes) == 4)
## 		dq.pts$altitude <- extract(altitudes[[1]], dq.pts)  # First element of altitudes covers 48 + DC
## 		success = 1
## 	}, warning = function(w) {
## 		Sys.sleep(30)
## 		counter = counter + 1
## 		if (counter< 10) success = 0
## 	}, error = function(e) {
## 		Sys.sleep(30)
## 		counter = counter + 1
## 		if (counter< 10) success = 0
## 	})
## }

message("Assigning altitudes from SRTM files")
dq.pts$srtm.altitude <- NA  # Will be populated (where possible) with values from SRTM files
dq.pts$srtm.filepath <- ""  # Path to SRTM file containing the point
dq.pts$srtm.cellnumber <- NA  # Cellnumber in corresponding SRTM file
dq.neighbors$srtm.altitude <- NA
srtm.files <- list.files(file.path(opt$data.dir, "SRTM", "SRTM2.1"))
srtm.files <- srtm.files[str_detect(srtm.files, "^N[0-9]{2}W[0-9]{3}\\.hgt$")]

# make list of srtm files in bounding rectangle
# this should work for finding the cells which cover the data as a rectangle
# in the lower 48 states, but data outside the lower 48 will break it
dqmat <- dq.pts@data
minlat <- min(dqmat[,"lat"])
minlon <- min(dqmat[,"lon"])
maxlat <- max(dqmat[,"lat"])
maxlon <- max(dqmat[,"lon"])

nmin <- floor(minlat)-1
nmax <- ceiling(maxlat)+1
wmin <- -floor(maxlon)-1
wmax <- -ceiling(minlon)+1

lappend <- function (lst, ...){
  lst <- c(lst, list(...))
  return(lst)
}

srtm.files = list()
for (w in wmin:wmax ) {
	for (n in nmin:nmax ) {
		if (n<10) {
			nstr = paste("0", toString(n), sep="")
		}
		else {
			nstr = toString(n)
		}
		if (w<100) {
			wstr = paste("0", toString(w), sep="")
		}
		else {
			wstr = toString(w)
		}
		srtmfile <- paste("N", nstr, "W", wstr, ".hgt", sep="")
		srtm.files <- lappend(srtm.files,srtmfile)
	}
}
srtm.files <- unlist(srtm.files)

message("Reading ", length(srtm.files), " SRTM files")
expected.file.size <- 25934402  # 2 * 3601^2 bytes

head(srtm.files)

for (file in srtm.files) {
    path <- file.path(opt$data.dir, "SRTM", "SRTM2.1", file, file)
    file.size <- file.info(path)$size    
	message("SRTM file ", path)
	if (!file.exists(path)) {
        message("SRTM file ", path, " is missing; skipping")
        next
    } else if (file.size != expected.file.size) {
        message("SRTM file ", path, " has unexpected size; skipping")
        message("actual=", format(file.size, big.mark=","),
                " expected=", format(expected.file.size, big.mark=","), " bytes")
        next
    } else {
        message("Reading altitude from ", path)
    }
    file.conn <- file(path, "rb")
    vec <- readBin(file.conn, "integer", n=3601*3601, size=2, signed=TRUE, endian="big")
    close(file.conn)
    vec <- ifelse(vec == -32768, NA, vec)  # "16-bit signed integers [...] with data voids indicated by -32768"
    stopifnot(length(vec) == 3601*3601)
    mat <- matrix(vec, nrow=3601, ncol=3601, byrow=TRUE)
    lat.bottom <- str_extract(file, "^N[0-9]{2}")
    lat.bottom <- as.integer(substr(lat.bottom, 2, 3))
    lon.left <- str_extract(file, "W[0-9]{3}")
    lon.left <- as.integer(substr(lon.left, 2, 4))
    srtm.raster <- raster(mat, xmn=-lon.left, xmx=(-lon.left + 1), ymn=lat.bottom, ymx=(lat.bottom + 1),
                          crs=CRS("+proj=longlat +datum=WGS84"))
    ## plot(srtm.raster)  # Sanity check
    extracted.values <- extract(srtm.raster, dq.pts, cellnumbers=TRUE)  # NAs for points outside current raster
    cellnumbers <- extracted.values[, 1]
    altitudes <- extracted.values[, 2]
    stopifnot(length(altitudes) == nrow(dq.pts))
    dq.pts$srtm.altitude[!is.na(altitudes)] <- altitudes[!is.na(altitudes)]
    dq.pts$srtm.cellnumber[!is.na(cellnumbers)] <- cellnumbers[!is.na(cellnumbers)]
    dq.pts$srtm.filepath[!is.na(altitudes)] <- path
    neighbor.altitudes <- extract(srtm.raster, dq.neighbors)
    dq.neighbors$srtm.altitude[!is.na(neighbor.altitudes)] <- neighbor.altitudes[!is.na(neighbor.altitudes)]
}
message("Fraction of dq.pts with non-NA srtm.altitude: ", round(mean(!is.na(dq.pts$srtm.altitude)), 4))
unique.srtm.cells <- length(unique(sprintf("%s %s", dq.pts$srtm.filepath, dq.pts$srtm.cellnumber)))  # Includes NAs
message("Unique SRTM cells relative to number of dq.pts: ", unique.srtm.cells, " / ", nrow(dq.pts),
        " = ", round(unique.srtm.cells / nrow(dq.pts), 4))

message("Calculating slope and aspect")
stopifnot(nrow(dq.neighbors) == 8 * nrow(dq.pts))
dq.pts$slope <- NA
dq.pts$aspect <- NA
for (i in 1:nrow(dq.pts)) {
    neighbors <- dq.neighbors[dq.neighbors$dqid == dq.pts$dqid[i], ]
    stopifnot(nrow(neighbors) == 8)
    altitudes <- as.data.frame(t(neighbors$srtm.altitude))
    names(altitudes) <- neighbors$label
    stopifnot(dim(altitudes) == c(1, 8))
    altitudes[is.na(altitudes)] <- dq.pts$srtm.altitude[i]
    if (any(is.na(altitudes))) {
        next  # Slope and aspect will be NA
    }
    size.x <- opt$neighbor.distance * cos(dq.pts$lat[i]* pi / 180) * 111.3 * 1000 / (60 * 60)
    dz.dx <- -(((altitudes$NW-altitudes$NE) + 2*(altitudes$W-altitudes$E) + (altitudes$SW-altitudes$SE)) /
               (8 * size.x))
    size.y <- opt$neighbor.distance * 111.6 * 1000 / (60 * 60)
    dz.dy <- (((altitudes$NW-altitudes$SW) + 2*(altitudes$N-altitudes$S) + (altitudes$NE-altitudes$SE)) /
              (8 * size.y))
    dq.pts$slope[i] <- atan(sqrt(dz.dx^2 + dz.dy^2))
    dq.pts$aspect[i] <- aspect(dz.dx, dz.dy)
}

message("Testing fn srid.from.lat.lon")
wg$srid.fit <- mapply(srid.from.lat.lon, lon=wg$lon, lat=wg$lat)
message("srid.from.lat.lon(lon=wg$lon, lat=wg$lat) is correct on all wg rows: ", all(round(wg$srid.fit) == wg$srid))
stopifnot(all(round(wg$srid.fit) == wg$srid))

message("Finding closest srid to each dq.pts")
dq.pts$srid.analytical <- closest.srid.analytical(lat=dq.pts$lat, lon=dq.pts$lon)
rerun.srid <- !dq.pts$srid.analytical %in% wg$srid
message("Need to run closest.srid.naive on ", sum(rerun.srid), " row(s) (",
        round(mean(rerun.srid), 4), " of total)")
if (any(rerun.srid)) {
    dq.pts$srid.analytical[rerun.srid] <- sapply(which(rerun.srid), closest.srid.naive)  # Careful, extremely slow
}

message("Writing results to ", opt$output.file)
write.csv(dq.pts@data, file=opt$output.file, row.names=FALSE)
