function [theta objval theta_hist Wopt] = twostage_gmm(theta0,dyccp,dyvc,dyvnc,x_ccp,x_vc,x_vnc,zccp,zvc,zvnc,dyccp_full,dyvc_full,dyvnc_full,x_ccp_full,x_vc_full,x_vnc_full,zccp_full,zvc_full,zvnc_full,fid,k,year,TT,W,wt,wt_full,slope,slope_full,keep,est,spec,ser)
% Calculate 2-stage GMM. First use a pre-specified weighting matrix,
% then use the optimal weighting matrix.
%
% Inputs:
% theta0     = value of parameters
% data       = dyccp,dyvc,dyvnc, x_ccp,x_vc,x_vnc, zccp,zvc,zvnc 
% data_full  = dyccp_full,dyvc_full,dyvnc_full, x_ccp_full,x_vc_full,x_vnc_full, zccp_full,zvc_full,zvnc_full
% fid        = field type
% k          = state variable k
% year       = years
% TT         = number of time periods
% W          = initial GMM Weighting matrix
% wt         = weigths for observations
% wt_full    = weigths for observations (full)
% slope      = slope variable
% slope_full = slope variable (full)
% keep       = select data
% est        = select moment restrictions:
% -est       = 1 is CCP
% -est       = 2 is V
% -est       = 3 is joint
% -est       = 4 is partial
% spec       = select model:
% -spec      = 1 uses no slope
% -spec      = 2 uses slope
% ser        = select the var-cov for optimal weighting matrix in 2nd stage
% -ser       = 1 uses standard robust var-cov
% -ser       = 2 uses clustered by year robust var-cov
% 
% Outputs:
% theta      = parameter that minimizes 2-stage GMM
% objval     = value of GMM criterion function at minimum
% theta_hist =
% Wopt       = optimal weighting matrix


% Options
options = optimset('Display','off','TolX',1e-13,'TolFun',1e-13,'MaxIter',200,'MaxFunEvals',500);

% Number of obs
N = size(dyccp,1);

% Sqrt of Weights
sqwt      = sqrt(wt);
sqwt_full = sqrt(wt_full);


% FIRST STAGE
% Set up to search for best theta 
theta_best = theta0(1,:);
obj_best   = 10^15;
theta_hist = [];

% Use Quasi-Newton method
for trial = 1:size(theta0,1)
	disp(['first stage of gmm, qn, trial ', num2str(trial)]);
	[theta1 obj1 eflag] = fminunc(@(thet) gmm_obj_full(thet,dyccp,dyvc,dyvnc,x_ccp, ...
							x_vc,x_vnc,zccp,zvc,zvnc,W,wt,slope,est,spec),theta0(trial,:), options);  
	if (obj1 < obj_best)
		theta_best = theta1;
		obj_best   = obj1;
	end
	theta_hist = [theta_hist theta1];
end

% Also try Nelder-Meade method
for trial = 1:size(theta0,1)
	disp(['first stage of gmm, nm, trial ', num2str(trial)]);
	[theta1 obj1 eflag] = fminsearch(@(thet) gmm_obj_full(thet,dyccp,dyvc,dyvnc,x_ccp, ...
							x_vc,x_vnc,zccp,zvc,zvnc,W,wt,slope,est,spec),theta0(trial,:), options);  
	if (obj1 < obj_best)
		theta_best = theta1;
		obj_best   = obj1;
	end
	theta_hist = [theta_hist theta1];
end

% PREPARE FOR SECOND STAGE
% Residuals from 1st stage 
if spec == 1
	sigma = 1/(theta_best);
	%eps1  = dyccp - x_ccp/sigma;
	%eps2  = dyvc  - x_vc  * sigma;
	%eps3  = dyvnc - x_vnc * sigma;
    eps1  = dyccp_full - x_ccp_full/sigma;
	eps2  = dyvc_full  - x_vc_full  * sigma;
	eps3  = dyvnc_full - x_vnc_full * sigma;
end
if spec == 2 
	%sigma = exp(theta_best(1) + theta_best(2)*slope);
	%eps1  = dyccp - x_ccp./sigma;
	%eps2  = dyvc  - x_vc  .* sigma;
	%eps3  = dyvnc - x_vnc .* sigma;
    sigma = exp(theta_best(1) + theta_best(2)*slope_full);
    eps1  = dyccp_full - x_ccp_full./sigma;
	eps2  = dyvc_full  - x_vc_full  .* sigma;
	eps3  = dyvnc_full - x_vnc_full .* sigma;
end


% Fitted moments
%gccp   = zccp .* eps1(:,ones(size(zccp,2),1)) .* wt(:,ones(size(zccp,2),1));
%gvc    = zvc  .* eps2(:,ones(size(zvc,2),1))  .* wt(:,ones(size(zvc,2),1));
%gvnc   = zvnc .* eps3(:,ones(size(zvnc,2),1)) .* wt(:,ones(size(zvnc,2),1));
%gccp   = zccp_full .* eps1(:,ones(size(zccp_full,2),1)) .* wt_full(:,ones(size(zccp_full,2),1));
%gvc    = zvc_full  .* eps2(:,ones(size(zvc_full,2),1))  .* wt_full(:,ones(size(zvc_full,2),1));
%gvnc   = zvnc_full .* eps3(:,ones(size(zvnc_full,2),1)) .* wt_full(:,ones(size(zvnc_full,2),1));

% Fitted moments (non-weighted)
gccp    = zccp_full .* eps1(:,ones(size(zccp_full,2),1));
gvc     = zvc_full  .* eps2(:,ones(size(zvc_full,2),1));
gvnc    = zvnc_full .* eps3(:,ones(size(zvnc_full,2),1));
% Fitted moments (weighted)
gccpw   = gccp .* wt_full(:,ones(size(zccp_full,2),1));
gvcw    = gvc  .* wt_full(:,ones(size(zvc_full,2),1));
gvncw   = gvnc .* wt_full(:,ones(size(zvnc_full,2),1));

% Get rid of 'NaN' 
gccp(isnan(gccp))   = 0;
gvc(isnan(gvc))     = 0;
gvnc(isnan(gvnc))   = 0;
gccpw(isnan(gccpw)) = 0;
gvcw(isnan(gvcw))   = 0;
gvncw(isnan(gvncw)) = 0;

% Select Moment Restrictions
if est == 1
	gfirst  = gccp;
    gfirstw = gccpw;
elseif est == 2
	gfirst  = [gvc,gvnc];
    gfirstw = [gvcw,gvncw];
elseif est == 3
	gfirst  = [gccp,gvc,gvnc];
    gfirstw = [gccpw,gvcw,gvncw];
elseif est == 4
	gfirst  = [gccp,gvc];
    gfirstw = [gccpw,gvcw];
end


% Optimal Weighting Matrix for 2nd-stage
% if ser = 1, use clustered-by-year robust var-cov (b/c of spatial dependence) 
if ser == 1
    % Inverse of Var-Cov Matrix
    Wopt = inv(gfirstw'*gfirstw/N);
end
if ser == 2
    % CHECK THIS!!! 
	%S    = calcS(gfirst,fid,k,year,wt,keep,TT);
    S    = calcS3(gfirst,fid,k,year,wt_full,keep,TT);
	if cond(S) > 10^5 
		S = eye(size(gfirst,2),size(gfirst,2));
	end
	Wopt = inv(S);
end
%if ser == 3   % this has not been checked
    % Nw   = sum(wt);
	%Sbasic = gfirst'*gfirst/Nw;
	%S = LWcov(gfirst,Sbasic);
	%Wopt = inv(S);
%end


% Starting Values, add best estimate to list of starting values
theta0 = [theta0; theta_best];

% SECOND STAGE
% Search for best theta using quasi-newton method
theta_best = theta0(1,:);
obj_best = 10^15;

% Use Quasi-Newton method
for trial = 1:size(theta0,1)
	disp(['second stage of gmm, qn, trial ', num2str(trial)]);
	[theta1 obj1 eflag] = fminunc(@(thet) gmm_obj_full(thet,dyccp,dyvc,dyvnc,x_ccp, ...
							x_vc,x_vnc,zccp,zvc,zvnc,Wopt,wt,slope,est,spec),theta0(trial,:), options);  
	if (obj1 < obj_best)
		theta_best = theta1;
		obj_best   = obj1;
	end
	theta_hist = [theta_hist theta1];
end

% Also try Nelder-Meade
for trial = 1:size(theta0,1)
	disp(['second stage of gmm, nm, trial ', num2str(trial)]);
	[theta1 obj1 eflag] = fminsearch(@(thet) gmm_obj_full(thet,dyccp,dyvc,dyvnc,x_ccp, ...
							x_vc,x_vnc,zccp,zvc,zvnc,Wopt,wt,slope,est,spec),theta0(trial,:), options);  
	if (obj1 < obj_best)
		theta_best = theta1;
		obj_best   = obj1;
	end
	theta_hist = [theta_hist theta1];
end

% Optimal Theta							
theta  = theta_best;

% Criterion Function
objval = obj_best;

